#include <stdio.h>
#include <functional>
#include <type_traits>
#include <typeinfo>

#define WR         /*read-write*/
#define RC         /*recursive*/
#define PS         /*program-state*/
#define API        /*namespace-api*/

#define interface  class
#define mixin      class

#define extends    public
#define implements public
#define imports    private
#define export     using
#define sealed     /*export-required*/
#define DI         /*dependency-injection*/

#define free       delete               // Free memory (not "delete variable")
#define with       std::function        // Method with External Method
#define cls        (*this)              // Calling Chain Link to Self
#define mix        (*this)              // Calling to Mix-in

#define at         class
#define on         class

#define type       typename
#define type_of    enable_if_same_type

#define include    if constexpr
#define PURE_C     1
#define STD_API    1
#define OS2_API    1
#define X11_API    1

#define size_of(x) ( sizeof (x) )
#define array_size(x) ( sizeof (x) )

#define t_id(x) ( typeid (x).hash_code () )
#define t_name(x) ( typeid (x).name () )

typedef void struct_ptr;

typedef unsigned char      byte;
typedef unsigned long      ulong;
typedef unsigned short     ushort;
typedef unsigned long long ulong64;

typedef unsigned long      hfile;
typedef unsigned long      hwnd;

template <typename T, typename U> inline constexpr
     int enable_if_same_type = std::is_same <T, U>::value;

/* * */

//  窠.
sealed namespace WindowManager
{
  interface IFrameWindow
  {
    // ...
  };

  interface IClientWindow
  {
    // ...
  };
}

export WindowManager::IFrameWindow;
export WindowManager::IClientWindow;

/* * */

// ࠡ稪 ᮮ饭.
sealed namespace MessageFilterInterface
{
  interface IMessageFilter
  {
    // ...
  };

  mixin TMessageFilter
  {
    // ...
  };
}

export MessageFilterInterface::IMessageFilter;
export MessageFilterInterface::TMessageFilter;

/* * */

// ᪫ .
sealed namespace LayoutManagerInterface
{
  interface ILayoutManager
  {
    // ...
  };

  mixin TLayoutManager
  {
    // ...
  };
}

export LayoutManagerInterface::ILayoutManager;
export LayoutManagerInterface::TLayoutManager;

/* * */

// .
sealed namespace FolderInterface
{
  template <interface F, interface C> interface AFolder
  {
    public:
      virtual WR AFolder &setTitle (char *) API = 0;
      virtual WR AFolder &setDocName (char *) API = 0;
    ;
  };
    
  template <interface F, interface C> interface IFolder : extends AFolder <F, C>
  {
    // ...
  };

  template <interface F, interface C> mixin WR TFolder
  {
    // ...
  };
}

export IFolder = FolderInterface::IFolder <IFrameWindow, IClientWindow>;
export TFolder = FolderInterface::TFolder <IFrameWindow, IClientWindow>;

/* * */

// 窠 ࠡ祣 ⮫.
sealed namespace WPShell
{
  class WR Folder : implements IFolder,
                       imports TFolder
  {
    public:
      Folder (char *title, IMessageFilter DI *filter,
                           ILayoutManager DI *layoutManager)
      {
        include (PURE_C | STD_API)
        {
          printf ("%s\n", title);
        }

        /*

        ... window = createWindow (...);
        ... window/localState/messageFilter = filter;
        ... window/localState/layoutManager = layoutManager;

        */

      }

      virtual ~Folder () {}

      IFolder &setTitle (char *);
      IFolder &setDocName (char *);
    ;

    private:
      hwnd window = 0;
    ;
  };

  IFolder &Folder::setTitle (char *title)
  {
    include (PURE_C)
    {
      printf ("%s\n", title);
    }

    return cls;
  }

  IFolder &Folder::setDocName (char *docName)
  {
    include (PURE_C)
    {
      printf ("%s\n", docName);
    }

    return cls;
  }
}

export WPShell::Folder;

/* * */

// 맮 API  ࠡ    䮫ࠬ.
namespace wm
{
  void show (IFrameWindow WR &window)
  {
    // ...
  }

  void hide (IFrameWindow WR &window)
  {
    // ...
  }
}

namespace wps
{
  void setTitle (IFolder WR &folder, char *title, char *docName = NULL)
  {
    char *_ = "";

    folder.setTitle (title).setDocName ((docName) ? docName : _);
  }
}

/* * */

// ﭨ ணࠬ (ᠭ ,   ᠬ ).
struct PS ProgramState
{
  Folder *folder = NULL;

  // ...

};

// ࠡ稪 ᮮ饭.
class MessageFilter : implements IMessageFilter,
                         imports TMessageFilter
{
  public:
    MessageFilter () {}
    virtual ~MessageFilter () {}
  ;

  /*

  private:
    ulong messages[] = {
                         { WM_MENUSELECT, MENU_ITEM_ID_ABOUT, MENU_ID_HELP },
                         ...
                       }; 

    #include "..."
  ;

  */

  private:
    class menu
    {
      at items
      {
        at Help
        {
          on About
          {
            public:
              ulong selected ( /* ... */ ProgramState *programState );
            ;
          };
        };
      };
    };

    // ...

  ;
};

ulong MessageFilter::menu::items::Help::About::selected ( /* ... */ ProgramState *programState )
{
  return 0;
}

class LayoutManager : implements ILayoutManager,
                         imports TLayoutManager
{
  public:
    LayoutManager () {}
    virtual ~LayoutManager () {}
  ;

  /*

  private:
    ulong messages[] = {
                         { WM_APPLYLAYOUT, 0, 0 },
                         ...
                       }; 

    #include "..."
  ;

  */

  // ...

};

// ---   䮫 ---

int main ()
{
  ProgramState *state = new ProgramState {0};

  state->folder = new Folder (
    "Hello, World!", new MessageFilter (),
                     new LayoutManager ()
  );

  wps::setTitle (*state->folder, "Folder", "Icon view");

  /*

  Application::current ().run (state);

  */

  return 0;
}
