#include <stdio.h>
#include <functional>
#include <type_traits>
#include <typeinfo>

#define WR         /*read-write*/
#define RC         /*recursive*/
#define PS         /*program-state*/
#define API        /*namespace-api*/

#define interface  class
#define mixin      class

#define extends    public
#define implements public
#define imports    private
#define export     using
#define sealed     /*export-required*/
#define DI         /*dependency-injection*/

#define free       delete               // Free memory (not "delete variable")
#define with       std::function        // Method with External Method
#define cls        (*this)              // Calling Chain Link to Self
#define mix        (*this)              // Calling to Mix-in

#define at         class
#define on         class

#define type       typename
#define type_of    enable_if_same_type

#define include    if constexpr
#define PURE_C     1
#define STD_API    1
#define OS2_API    1
#define X11_API    1

#define size_of(x) ( sizeof (x) )
#define array_size(x) ( sizeof (x) )

#define t_id(x) ( typeid (x).hash_code () )
#define t_name(x) ( typeid (x).name () )

typedef void struct_ptr;

typedef unsigned char      byte;
typedef unsigned long      ulong;
typedef unsigned short     ushort;
typedef unsigned long long ulong64;

typedef unsigned long      hfile;
typedef unsigned long      hwnd;

template <typename T, typename U> inline constexpr
     int enable_if_same_type = std::is_same <T, U>::value;

/* * */

// ࠭  窨.
sealed namespace PaintingToolbox
{
  // ஡  ⥩.
  template <class T> interface IHolder
  {
    public:
      virtual void WR add (ulong, T &) = 0;
      virtual void WR remove (ulong) = 0;
   
      virtual int isEmpty () = 0;
    ;
  };

  // ஡  ⥩.
  template <class T> interface ITHolder
  {
    protected:
      virtual int checkIsEmpty () = 0;
    ;
  };

  // ஡  ⥩.
  template <class T> mixin WR THolder : implements ITHolder <T>
  {
    public:
      THolder () {}
      ~THolder () {}
    ;

    protected:
      void WR incCounter ();
      void WR decCounter ();

      int checkIsEmpty ();
    ;

    private:
      ulong counter = 0;
    ;
  };

  template <class T> void THolder <T>::incCounter ()
  {
    counter ++;
  }

  template <class T> void THolder <T>::decCounter ()
  {
    counter --;
  }

  template <class T> int THolder <T>::checkIsEmpty ()
  {
    return (counter == 0) ? 1 : 0;
  }

  // ।  쬠.
  interface IDrawStick
  {
    public:
      virtual void draw () = 0;
    ;
  };

  // ࠭.
  class Pencil : implements IDrawStick 
  {
    public:
      Pencil () {}
      virtual ~Pencil () {}
 
      void draw ();
    ;
  };

  void Pencil::draw ()
  {
    include (PURE_C)
    {
      printf ("Pencil::draw ()\n");
    }
  }

  // 窠.
  class Brush : implements IDrawStick
  {
    public:
      Brush () {}
      virtual ~Brush () {}
    
      void draw ();
    ;
  };

  void Brush::draw ()
  {
    include (PURE_C)
    {
      printf ("Brush::draw ()\n");
    }
  }

  // ⥩  祬 㣮.
  template <class T> class WR Container : implements IHolder <T>, 
                                             imports THolder <T>
  {
    public:
      Container () {}
      virtual ~Container () {}

      void add (ulong, T &);
      void remove (ulong);

      int isEmpty ();
    ;
  };

  template <class T> void Container <T>::add (ulong id, T &item)
  {
    include (PURE_C)
    {
      printf ("Container::add ()\n");
    }

    mix.incCounter ();
  }

  template <class T> void Container <T>::remove (ulong id)
  {
    include (PURE_C)
    {
      printf ("Container::remove ()\n");
    }

    mix.decCounter ();
  }

  template <class T> int Container <T>::isEmpty ()
  {
    include (PURE_C)
    {
      printf ("Container::isEmpty ()\n");
    }

    return mix.checkIsEmpty ();
  }

  // ஡  祬 㣮.
  template <class T> class WR Box : implements IHolder <T>, 
                                       imports THolder <T>
  {
    public:
      Box () {}
      virtual ~Box () {}

      void add (ulong, T &);
      void remove (ulong);

      int isEmpty ();
    ;
  };

  template <class T> void Box <T>::add (ulong id, T &item)
  {
    include (PURE_C)
    {
      printf ("Box::add ()\n");
    }

    mix.incCounter ();
  }

  template <class T> void Box <T>::remove (ulong id)
  {
    include (PURE_C)
    {
      printf ("Box::remove ()\n");
    }

    mix.decCounter ();
  }

  template <class T> int Box <T>::isEmpty ()
  {
    include (PURE_C)
    {
      printf ("Box::isEmpty ()\n");
    }

    return mix.checkIsEmpty ();
  }
}

export PaintingToolbox::IDrawStick;
export PaintingToolbox::Pencil;
export PaintingToolbox::Brush;
export PaintingToolbox::Container;
export PaintingToolbox::Box;

/* * */

int main ()
{
  // ࠭.
  Pencil *pencil = new Pencil ();

  // 窠.
  IDrawStick *brush = new Brush ();

  // ஡  ࠭蠬.
  Box <Pencil> *pencilBox = new Box <Pencil> ();

  // ⥩  ஡  ࠭蠬  窠.
  Container <Box <IDrawStick>> *container = new Container <Box <IDrawStick>> ();

  // ...
  pencil->draw ();
  brush->draw ();

  pencilBox->add (0xFF, *pencil);

  include (PURE_C)
  {
    printf ("%d\n", pencilBox->isEmpty ());
    printf ("%d\n", container->isEmpty ());
  }
    
  //  , 祬 ஡.     
        return 0;
}
